'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var strictUriEncode = function (str) {
	return encodeURIComponent(str).replace(/[!'()*]/g, function (c) {
		return '%' + c.charCodeAt(0).toString(16).toUpperCase();
	});
};

/*
object-assign
(c) Sindre Sorhus
@license MIT
*/
/* eslint-disable no-unused-vars */
var getOwnPropertySymbols = Object.getOwnPropertySymbols;
var hasOwnProperty = Object.prototype.hasOwnProperty;
var propIsEnumerable = Object.prototype.propertyIsEnumerable;

function toObject(val) {
	if (val === null || val === undefined) {
		throw new TypeError('Object.assign cannot be called with null or undefined');
	}

	return Object(val);
}

function shouldUseNative() {
	try {
		if (!Object.assign) {
			return false;
		}

		// Detect buggy property enumeration order in older V8 versions.

		// https://bugs.chromium.org/p/v8/issues/detail?id=4118
		var test1 = new String('abc');  // eslint-disable-line no-new-wrappers
		test1[5] = 'de';
		if (Object.getOwnPropertyNames(test1)[0] === '5') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test2 = {};
		for (var i = 0; i < 10; i++) {
			test2['_' + String.fromCharCode(i)] = i;
		}
		var order2 = Object.getOwnPropertyNames(test2).map(function (n) {
			return test2[n];
		});
		if (order2.join('') !== '0123456789') {
			return false;
		}

		// https://bugs.chromium.org/p/v8/issues/detail?id=3056
		var test3 = {};
		'abcdefghijklmnopqrst'.split('').forEach(function (letter) {
			test3[letter] = letter;
		});
		if (Object.keys(Object.assign({}, test3)).join('') !==
				'abcdefghijklmnopqrst') {
			return false;
		}

		return true;
	} catch (err) {
		// We don't expect any of the above to throw, but better to be safe.
		return false;
	}
}

var objectAssign = shouldUseNative() ? Object.assign : function (target, source) {
	var from;
	var to = toObject(target);
	var symbols;

	for (var s = 1; s < arguments.length; s++) {
		from = Object(arguments[s]);

		for (var key in from) {
			if (hasOwnProperty.call(from, key)) {
				to[key] = from[key];
			}
		}

		if (getOwnPropertySymbols) {
			symbols = getOwnPropertySymbols(from);
			for (var i = 0; i < symbols.length; i++) {
				if (propIsEnumerable.call(from, symbols[i])) {
					to[symbols[i]] = from[symbols[i]];
				}
			}
		}
	}

	return to;
};

var token = '%[a-f0-9]{2}';
var singleMatcher = new RegExp(token, 'gi');
var multiMatcher = new RegExp('(' + token + ')+', 'gi');

function decodeComponents(components, split) {
	try {
		// Try to decode the entire string first
		return decodeURIComponent(components.join(''));
	} catch (err) {
		// Do nothing
	}

	if (components.length === 1) {
		return components;
	}

	split = split || 1;

	// Split the array in 2 parts
	var left = components.slice(0, split);
	var right = components.slice(split);

	return Array.prototype.concat.call([], decodeComponents(left), decodeComponents(right));
}

function decode(input) {
	try {
		return decodeURIComponent(input);
	} catch (err) {
		var tokens = input.match(singleMatcher);

		for (var i = 1; i < tokens.length; i++) {
			input = decodeComponents(tokens, i).join('');

			tokens = input.match(singleMatcher);
		}

		return input;
	}
}

function customDecodeURIComponent(input) {
	// Keep track of all the replacements and prefill the map with the `BOM`
	var replaceMap = {
		'%FE%FF': '\uFFFD\uFFFD',
		'%FF%FE': '\uFFFD\uFFFD'
	};

	var match = multiMatcher.exec(input);
	while (match) {
		try {
			// Decode as big chunks as possible
			replaceMap[match[0]] = decodeURIComponent(match[0]);
		} catch (err) {
			var result = decode(match[0]);

			if (result !== match[0]) {
				replaceMap[match[0]] = result;
			}
		}

		match = multiMatcher.exec(input);
	}

	// Add `%C2` at the end of the map to make sure it does not replace the combinator before everything else
	replaceMap['%C2'] = '\uFFFD';

	var entries = Object.keys(replaceMap);

	for (var i = 0; i < entries.length; i++) {
		// Replace all decoded components
		var key = entries[i];
		input = input.replace(new RegExp(key, 'g'), replaceMap[key]);
	}

	return input;
}

var decodeUriComponent = function (encodedURI) {
	if (typeof encodedURI !== 'string') {
		throw new TypeError('Expected `encodedURI` to be of type `string`, got `' + typeof encodedURI + '`');
	}

	try {
		encodedURI = encodedURI.replace(/\+/g, ' ');

		// Try the built in decoder first
		return decodeURIComponent(encodedURI);
	} catch (err) {
		// Fallback to a more advanced decoder
		return customDecodeURIComponent(encodedURI);
	}
};

function encoderForArrayFormat(opts) {
	switch (opts.arrayFormat) {
		case 'index':
			return function (key, value, index) {
				return value === null ? [
					encode(key, opts),
					'[',
					index,
					']'
				].join('') : [
					encode(key, opts),
					'[',
					encode(index, opts),
					']=',
					encode(value, opts)
				].join('');
			};

		case 'bracket':
			return function (key, value) {
				return value === null ? encode(key, opts) : [
					encode(key, opts),
					'[]=',
					encode(value, opts)
				].join('');
			};

		default:
			return function (key, value) {
				return value === null ? encode(key, opts) : [
					encode(key, opts),
					'=',
					encode(value, opts)
				].join('');
			};
	}
}

function parserForArrayFormat(opts) {
	var result;

	switch (opts.arrayFormat) {
		case 'index':
			return function (key, value, accumulator) {
				result = /\[(\d*)\]$/.exec(key);

				key = key.replace(/\[\d*\]$/, '');

				if (!result) {
					accumulator[key] = value;
					return;
				}

				if (accumulator[key] === undefined) {
					accumulator[key] = {};
				}

				accumulator[key][result[1]] = value;
			};

		case 'bracket':
			return function (key, value, accumulator) {
				result = /(\[\])$/.exec(key);
				key = key.replace(/\[\]$/, '');

				if (!result) {
					accumulator[key] = value;
					return;
				} else if (accumulator[key] === undefined) {
					accumulator[key] = [value];
					return;
				}

				accumulator[key] = [].concat(accumulator[key], value);
			};

		default:
			return function (key, value, accumulator) {
				if (accumulator[key] === undefined) {
					accumulator[key] = value;
					return;
				}

				accumulator[key] = [].concat(accumulator[key], value);
			};
	}
}

function encode(value, opts) {
	if (opts.encode) {
		return opts.strict ? strictUriEncode(value) : encodeURIComponent(value);
	}

	return value;
}

function keysSorter(input) {
	if (Array.isArray(input)) {
		return input.sort();
	} else if (typeof input === 'object') {
		return keysSorter(Object.keys(input)).sort(function (a, b) {
			return Number(a) - Number(b);
		}).map(function (key) {
			return input[key];
		});
	}

	return input;
}

function extract(str) {
	var queryStart = str.indexOf('?');
	if (queryStart === -1) {
		return '';
	}
	return str.slice(queryStart + 1);
}

function parse(str, opts) {
	opts = objectAssign({arrayFormat: 'none'}, opts);

	var formatter = parserForArrayFormat(opts);

	// Create an object with no prototype
	// https://github.com/sindresorhus/query-string/issues/47
	var ret = Object.create(null);

	if (typeof str !== 'string') {
		return ret;
	}

	str = str.trim().replace(/^[?#&]/, '');

	if (!str) {
		return ret;
	}

	str.split('&').forEach(function (param) {
		var parts = param.replace(/\+/g, ' ').split('=');
		// Firefox (pre 40) decodes `%3D` to `=`
		// https://github.com/sindresorhus/query-string/pull/37
		var key = parts.shift();
		var val = parts.length > 0 ? parts.join('=') : undefined;

		// missing `=` should be `null`:
		// http://w3.org/TR/2012/WD-url-20120524/#collect-url-parameters
		val = val === undefined ? null : decodeUriComponent(val);

		formatter(decodeUriComponent(key), val, ret);
	});

	return Object.keys(ret).sort().reduce(function (result, key) {
		var val = ret[key];
		if (Boolean(val) && typeof val === 'object' && !Array.isArray(val)) {
			// Sort object keys, not values
			result[key] = keysSorter(val);
		} else {
			result[key] = val;
		}

		return result;
	}, Object.create(null));
}

var extract_1 = extract;
var parse_1 = parse;

var stringify = function (obj, opts) {
	var defaults = {
		encode: true,
		strict: true,
		arrayFormat: 'none'
	};

	opts = objectAssign(defaults, opts);

	if (opts.sort === false) {
		opts.sort = function () {};
	}

	var formatter = encoderForArrayFormat(opts);

	return obj ? Object.keys(obj).sort(opts.sort).map(function (key) {
		var val = obj[key];

		if (val === undefined) {
			return '';
		}

		if (val === null) {
			return encode(key, opts);
		}

		if (Array.isArray(val)) {
			var result = [];

			val.slice().forEach(function (val2) {
				if (val2 === undefined) {
					return;
				}

				result.push(formatter(key, val2, result.length));
			});

			return result.join('&');
		}

		return encode(key, opts) + '=' + encode(val, opts);
	}).filter(function (x) {
		return x.length > 0;
	}).join('&') : '';
};

var parseUrl = function (str, opts) {
	return {
		url: str.split('?')[0] || '',
		query: parse(extract(str), opts)
	};
};

var queryString = {
	extract: extract_1,
	parse: parse_1,
	stringify: stringify,
	parseUrl: parseUrl
};

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-restricted-syntax */
var Bound = function Bound() {
  _classCallCheck(this, Bound);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.getOwnPropertyNames(this.constructor.prototype)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      var value = this[key];

      if (key !== 'constructor' && typeof value === 'function') {
        this[key] = value.bind(this);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
};

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function join() {
  var result = [];

  // If the first part is a plain protocol, we combine it with the next part.

  for (var _len = arguments.length, parts = Array(_len), _key = 0; _key < _len; _key++) {
    parts[_key] = arguments[_key];
  }

  if (parts[0].match(/^[^/:]+:\/*$/) && parts.length > 1) {
    var first = parts.shift();
    parts[0] = first + parts[0];
  }

  // There must be two or three slashes in the file protocol, two slashes in anything else.
  if (parts[0].match(/^file:\/\/\//)) {
    parts[0] = parts[0].replace(/^([^/:]+):\/*/, '$1:///');
  } else {
    parts[0] = parts[0].replace(/^([^/:]+):\/*/, '$1://');
  }

  for (var i = 0; i < parts.length; i += 1) {
    var component = parts[i];

    if (typeof component !== 'string') {
      throw new TypeError('Url must be a string. Received ' + component);
    }

    // eslint-disable-next-line no-continue
    if (component === '') continue;

    if (i > 0) {
      // Removing the starting slashes for each component but the first.
      component = component.replace(/^[/]+/, '');
    }
    if (i < parts.length - 1) {
      // Removing the ending slashes for each component but the last.
      component = component.replace(/[/]+$/, '');
    } else {
      // For the last component we will combine multiple slashes to a single one.
      component = component.replace(/[/]+$/, '/');
    }

    result.push(component);
  }

  var str = result.join('/');
  // Each input component is now separated by a single slash except the possible
  // first plain protocol part.

  // remove trailing slash before parameters or hash
  str = str.replace(/\/(\?|&|#[^!])/g, '$1');

  // replace ? in parameters with &
  var params = str.split('?');
  str = params.shift() + (params.length > 0 ? '?' : '') + params.join('&');

  return str;
}

function has(value, key) {
  return Object.prototype.hasOwnProperty.call(value, key);
}

var VALID_TYPEOF = ['string', 'undefined', 'object', 'boolean', 'null', 'number', 'function', 'array'];

function checkType(value, type) {
  if (typeof type !== 'string') {
    throw new TypeError('`type` argument must be a string, got ' + (typeof type === 'undefined' ? 'undefined' : _typeof(type)));
  }
  if (VALID_TYPEOF.indexOf(type) === -1) {
    throw new TypeError('type argument must be one of ' + VALID_TYPEOF.join(','));
  }

  switch (type) {
    case 'null':
      return value === null;
    case 'array':
      return Array.isArray(value);
    case 'object':
      return value !== null && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === type; // eslint-disable-line valid-typeof
    default:
      return (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === type; // eslint-disable-line valid-typeof
  }
}

function warn(condition, message) {
  if (condition) {
    var _console;

    for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
      args[_key2 - 2] = arguments[_key2];
    }

    (_console = console).warn.apply(_console, [message].concat(args)); // eslint-disable-line no-console
  }
}

function isId(value) {
  var int = parseInt(value, 10);

  if (typeof value === 'number') return value === int;
  return !isNaN(int);
}

function invariant(condition, format) {
  for (var _len3 = arguments.length, args = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
    args[_key3 - 2] = arguments[_key3];
  }

  if (!condition) {
    var argIndex = 0;
    var message = format.replace(/%s/g, function () {
      return args[argIndex++];
    });
    var error = new Error(message);
    error.name = 'Invariant violation';
    throw error;
  }
}

function isPlainObject(value) {
  return value && value.constructor === Object;
}

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Class for creating an SDK for the Aula Mejor API with
 * with base headers and URL
 */

var AulaMejorClient = function (_Bound) {
  _inherits(AulaMejorClient, _Bound);

  /**
   * Create an API client
   * @param {object} options - Client configuration
   * @property {string} options.baseURL - Base url for all the requests
   * @property {string} [options.bearerToken] - Bearer token to send in requests that require authorization
   */
  function AulaMejorClient() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck$1(this, AulaMejorClient);

    var _this = _possibleConstructorReturn(this, (AulaMejorClient.__proto__ || Object.getPrototypeOf(AulaMejorClient)).call(this));

    if (!has(options, 'baseURL')) {
      throw new TypeError('baseURL argument is required');
    }

    if (typeof options.baseURL !== 'string') {
      throw new TypeError('baseURL argument must be a string');
    }

    _this.baseURL = options.baseURL;
    _this.headers = new Headers({ Accept: 'application/json' });
    _this.bearerToken = options.bearerToken;
    return _this;
  }

  _createClass(AulaMejorClient, [{
    key: 'request',
    value: function request(_ref) {
      var bearer = _ref.bearer,
          url = _ref.url,
          options = _objectWithoutProperties(_ref, ['bearer', 'url']);

      // We need to add an empty object because Edge
      // doesn't allows undefined as an argument
      var headers = new Headers(options.headers || {});

      this.headers.forEach(function (value, key) {
        headers.append(key, value);
      });

      if (bearer) {
        headers.set('Authorization', 'Bearer ' + this.bearerToken);
      }

      if (options.method === 'POST' || options.method === 'PUT' || options.method === 'PATCH') {
        // If the request body is a JSON object stringify it
        if (isPlainObject(options.body)) {
          var body = {
            payload: options.body
          };
          options.body = JSON.stringify(body);
          headers.set('Content-Type', 'application/json');
        }
      }

      var uri = join(this.baseURL, url);
      var request = new Request(uri, _extends({}, options, { headers: headers }));
      var responsePromise = fetch(request);
      var jsonPromise = responsePromise.then(function (res) {
        return res.json();
      }).catch(function (e) {
        if (e.name !== 'SyntaxError') {
          throw e;
        }
      });

      return Promise.all([responsePromise, jsonPromise]).then(function (_ref2) {
        var _ref3 = _slicedToArray(_ref2, 2),
            response = _ref3[0],
            json = _ref3[1];

        if (response.status >= 200 && response.status < 300) {
          return json && json.payload;
        }

        // Transform response into an error instance
        var error = new Error(json.message);
        error.statusCode = response.status;
        error.name = json.code;
        throw error;
      });
    }
  }, {
    key: 'get',
    value: function get(url) {
      var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var params = _ref4.params,
          options = _objectWithoutProperties(_ref4, ['params']);

      var query = queryString.stringify(params);

      if (query.length) {
        query = '?' + query;
      }

      var uri = join(url, query);

      return this.request(_extends({}, options, { url: uri, method: 'GET' }));
    }
  }, {
    key: 'post',
    value: function post(url, options) {
      return this.request(_extends({}, options, {
        url: url,
        method: 'POST'
      }));
    }
  }, {
    key: 'patch',
    value: function patch(url, options) {
      return this.request(_extends({}, options, { url: url, method: 'PATCH' }));
    }
  }, {
    key: 'checkToken',
    value: function checkToken() {
      if (typeof this.bearerToken !== 'string') {
        throw new TypeError('bearer token must be a string, got ' + _typeof$1(this.bearerToken));
      }
    }
  }, {
    key: 'checkArgumentType',
    value: function checkArgumentType(value, name, type) {
      if (Array.isArray(type)) {
        if (!type.some(function (t) {
          return checkType(value, t);
        })) {
          var types = type.join(' or ');
          throw new TypeError(name + ' argument must be a ' + types + ', got ' + (typeof value === 'undefined' ? 'undefined' : _typeof$1(value)));
        }
      } else if (!checkType(value, type)) {
        throw new TypeError(name + ' argument must be a ' + type + ', got ' + (typeof value === 'undefined' ? 'undefined' : _typeof$1(value)));
      }
    }

    /**
     * Create a new user
     * @param {object} params - Date to create the new user
     * @property {string} email - User email
     * @property {string} password - The user password in plain text
     * @property {string} firstName - First name of the user
     * @property {string} lastName - Last name of the user
     * @return {object} - Date of the created user
     */

  }, {
    key: 'registerUser',
    value: function registerUser(_ref5) {
      var email = _ref5.email,
          password = _ref5.password,
          firstName = _ref5.firstName,
          lastName = _ref5.lastName;

      this.checkArgumentType(email, 'email', 'string');
      this.checkArgumentType(password, 'password', 'string');
      this.checkArgumentType(firstName, 'firstName', 'string');
      this.checkArgumentType(lastName, 'lastName', 'string');

      return this.post('/users', {
        body: {
          email: email,
          password: password,
          firstName: firstName,
          lastName: lastName
        }
      });
    }

    /**
     * Retrieve the user associated with the current access token
     * @return {object} - Data for the current user
     */

  }, {
    key: 'getCurrentUser',
    value: function getCurrentUser() {
      this.checkToken();

      return this.get('/user', {
        bearer: true
      });
    }

    /**
     * Sends an email to the given address to recover password
     * @param {string} email - Email address associated of the user
     */

  }, {
    key: 'recoverPassword',
    value: function recoverPassword(email) {
      this.checkArgumentType(email, 'email', 'string');

      return this.post('/users/send_password_change', {
        body: {
          email: email
        }
      });
    }

    /**
     * Sends an account verification email to the given address
     * @param {string} email - Email address associated of the user
     */

  }, {
    key: 'sendEmailVerification',
    value: function sendEmailVerification(email) {
      this.checkArgumentType(email, 'email', 'string');

      return this.post('/users/send_verification', {
        body: {
          email: email
        }
      });
    }

    /**
     * Confirms a users email verification
     * @param {string} email - Email address associated of the user
     */

  }, {
    key: 'verifyEmail',
    value: function verifyEmail(email) {
      this.checkArgumentType(email, 'email', 'string');

      return this.post('/users/verify_email', {
        body: {
          email: email
        }
      });
    }

    /**
     * Retrieves a list of the 7 must recent courses
     * @deprecated
     * @return {array} - Array of course objects
     */

  }, {
    key: 'getRecentCourses',
    value: function getRecentCourses() {
      warn(true, "getRecentCourses() has been deprecated in favor of getCourses({ scope: 'recent' }) and will be removed in the next major version");

      return this.get('/programs', {
        params: {
          scope: 'recent'
        }
      });
    }

    /**
     * Gets the static content for a given section key
     * @param {string} section - Section key
     */

  }, {
    key: 'getStaticContent',
    value: function getStaticContent(sections) {
      this.checkArgumentType(sections, 'sections', ['string', 'array']);

      return this.get('/static_content', {
        params: {
          sections: Array.isArray(sections) ? sections.join(',') : sections
        }
      });
    }

    /**
     * Getters and setters for bearer token
     */

  }, {
    key: 'getFeaturedCourses',


    /**
     * Retrieves a list of featured courses
     * @deprecated
     * @return {array} - Array of course objects
     */
    value: function getFeaturedCourses() {
      warn(true, "getRecentCourses() has been deprecated in favor of getCourses({ scope: 'recent' }) and will be removed in the next major version");

      return this.get('/programs', {
        params: {
          scope: 'featured'
        }
      });
    }

    /**
     * Retrieves a list of courses with a given scope, page and page limit
     * @param {options} options - Options of the request
     * @property {string} [options.scope] - A valid scope for the programs resource
     * @property {number} [options.page] - A page number
     * @property {number} [options.limit] - A limit number used to truncate a page.
     * @property {boolean} [options.next] - True to return the next page when page has been requested
     * @return {array} - Array of courses
     */

  }, {
    key: 'getCourses',
    value: function getCourses() {
      var _this2 = this;

      var _ref6 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          scope = _ref6.scope,
          page = _ref6.page,
          limit = _ref6.limit,
          url = _ref6.url,
          _ref6$next = _ref6.next,
          next = _ref6$next === undefined ? false : _ref6$next;

      if (!scope && !page && !limit && !url && !next) {
        throw new TypeError('You must provide either a scope or a page, when next is set to false');
      }

      if (scope) this.checkArgumentType(scope, 'scope', 'string');
      if (page) this.checkArgumentType(page, 'page', 'number');
      if (limit) this.checkArgumentType(limit, 'limit', 'number');
      if (url) this.checkArgumentType(url, 'url', 'string');

      var response = void 0;

      if (next && this.nextCoursePage) {
        response = this.get(this.nextCoursePage);
      } else {
        if (next) {
          page = 1;
        }

        response = this.get('/programs', {
          params: {
            scope: scope,
            page: page,
            limit: limit,
            url: url
          }
        });
      }

      return response.then(function (payload) {
        if (payload.next) {
          _this2.nextCoursePage = payload.next;
        }

        return payload.programs || payload;
      });
    }

    /**
     * Get the groups associated with a course
     * @param {number} courseId - Id of the course to retrieve
     * @return {Array} - An array of groups
     */

  }, {
    key: 'getCourseGroups',
    value: function getCourseGroups(courseId) {
      var _ref7 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          scope = _ref7.scope;

      invariant(isId(courseId), 'Invalid id ' + courseId + ', id must be an integer');
      if (scope) this.checkArgumentType(scope, 'scope', 'string');

      return this.get('/programs/' + courseId + '/groups', {
        params: {
          scope: scope
        }
      });
    }

    /**
     * Get the requirements for signin in a given group
     * @param {number} groupId
     * @return {Array}
     */

  }, {
    key: 'getGroupRequirements',
    value: function getGroupRequirements(groupId) {
      invariant(isId(groupId), 'Invalid id ' + groupId + ', id must be an integer');

      return this.get('/groups/' + groupId + '/requirements');
    }

    /**
     * Send a user question via email
     * @param {object} args
     * @param {string} args.email - The user email
     * @param {string} args.name - Name of the user
     * @param {string} args.course - The course the user wants to ask about
     */

  }, {
    key: 'sendUserQuestion',
    value: function sendUserQuestion() {
      var _ref8 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          name = _ref8.name,
          email = _ref8.email,
          course = _ref8.course;

      warn(true, "sendUserQuestion() has been deprecated in favor of sendEmail({ name: 'ask_info' }) and will be removed in the next major version");

      var data = new FormData();

      data.set('email', email);
      data.set('name', name);
      data.set('course', course);

      return this.sendEmail({ name: 'request-information', data: data });
    }

    /**
     * Send user info from any form via email
     * @param {object} args
     * @param {string} args.name - Name of the template
     * @param {object} args.data - The data user will send
     */

  }, {
    key: 'sendEmail',
    value: function sendEmail() {
      var _ref9 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          name = _ref9.name,
          data = _ref9.data;

      this.checkArgumentType(name, 'name', 'string');
      this.checkArgumentType(data, 'data', 'object');

      return this.post('emails/' + name, { body: data });
    }

    /**
     * Count the number of programs optinally grouped by some form of response (i.e. pages)
     * @param {object} args
     * @param {string} args.by - Response form to group the count by
     * @param {number} args.limit - Page limit, this will only be taken into account for { by: "page" }
     */

  }, {
    key: 'countCourses',
    value: function countCourses() {
      var _ref10 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          by = _ref10.by,
          limit = _ref10.limit;

      if (by) this.checkArgumentType(by, 'by', 'string');
      if (limit) this.checkArgumentType(limit, 'limit', 'number');

      return this.get('/programs/count', {
        params: {
          by: by,
          limit: limit
        }
      });
    }

    /**
     * Get the course agenda witth the program id as the request
     * @param {number} programId - Id of the program to retrieve
     */

  }, {
    key: 'getCourseAgenda',
    value: function getCourseAgenda(programId) {
      invariant(isId(programId), 'Invalid id ' + programId + ', id must be an integer');

      return this.get('/programs/' + programId + '/subjects');
    }

    /**
     * Get an emitted diploma by id
     * @param {string} id - Id of the diploma
     * @param {string} scope - Scope to fetch
     */

  }, {
    key: 'getDiploma',
    value: function getDiploma(id) {
      var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'validation';

      this.checkArgumentType(id, 'id', 'string');

      return this.get('/diplomas/' + id, {
        params: {
          scope: scope
        }
      });
    }

    /**
     * Get help categories
     * @return {array} Array of help categories
     */

  }, {
    key: 'getHelpCategories',
    value: function getHelpCategories() {
      return this.get('/help-categories');
    }

    /**
     * Create a question in the help knowledge base
     * @param {object} args - Question to create
     * @param {number} args.order - Order of the question with respect to the other questions
     * @param {string} args.question - The actual question
     * @param {number} args.helpCategoryId - Id of the category the question will be included in
     * @param {boolean} [args.isVisible=true] - True if the question will show up in queries to de API
     */

  }, {
    key: 'createHelpQuestion',
    value: function createHelpQuestion() {
      var _ref11 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          order = _ref11.order,
          question = _ref11.question,
          helpCategoryId = _ref11.helpCategoryId,
          _ref11$isVisible = _ref11.isVisible,
          isVisible = _ref11$isVisible === undefined ? true : _ref11$isVisible;

      return this.post('/help-questions', {
        body: {
          order: order,
          question: question,
          helpCategoryId: helpCategoryId,
          isVisible: isVisible
        }
      });
    }

    /**
     * Get the most requested topics
     * @return {array} Array of the most requested topics
     */

  }, {
    key: 'getHelpQuestionsByScope',
    value: function getHelpQuestionsByScope(scope) {
      if (scope) this.checkArgumentType(scope, 'scope', 'string');

      return this.get('/help-questions', {
        params: {
          scope: scope
        }
      });
    }

    /**
     * Get Get help answers belonging to a question
     * @param {number} questionId Id of the question
     * @return {Promise<Array<Object>>} Promise that resolves to an array of answer objects
     */

  }, {
    key: 'getHelpQuestionAnswers',
    value: function getHelpQuestionAnswers(questionId) {
      invariant(isId(questionId), 'Invalid id ' + questionId + ', id must be an integer');

      return this.get('/help-questions/' + questionId + '/answers');
    }

    /**
     * Get the questions by category in contact section
     * @param {number} categoryId - Id of the category
     */

  }, {
    key: 'getQuestionsByCategoryId',
    value: function getQuestionsByCategoryId(categoryId) {
      invariant(isId(categoryId), 'Invalid id ' + categoryId + ', id must be an integer');

      return this.get('/help-categories/' + categoryId + '/questions');
    }

    /**
     * Get payment methods
     * @returns {array} Array of payment methods
     */

  }, {
    key: 'getPaymentMethods',
    value: function getPaymentMethods() {
      return this.get('/payment_methods');
    }

    /**
     * Create a payment for a course
     *
     * @param {Object} payload - Data to create a new payment
     * @param {Array} payload.price - Price of the course to pay
     * @param {number} payload.enrollmentId - Id of enrollment the payment belongs to
     * @param {number} payload.paymentMethodId - Id of payment method to be used
     * @param {Object} [payload.redirectURLs] - URLs of success payment and cancel payments
     */

  }, {
    key: 'createPayment',
    value: function createPayment() {
      var _ref12 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          price = _ref12.price,
          paymentMethodId = _ref12.paymentMethodId,
          enrollmentId = _ref12.enrollmentId,
          redirectURLs = _ref12.redirectURLs;

      this.checkArgumentType(price, 'price', 'number');
      this.checkArgumentType(paymentMethodId, 'paymentMethodId', 'number');
      this.checkArgumentType(enrollmentId, 'enrollmentId', 'number');
      this.checkArgumentType(redirectURLs, 'redirectURLs', 'object');

      return this.post('/payments', {
        body: {
          price: price,
          paymentMethodId: paymentMethodId,
          enrollmentId: enrollmentId,
          redirectURLs: redirectURLs
        },
        bearer: true
      });
    }

    /**
     * Post student
     *
     * @param {Object} params - Data to create new enrollment
     * @param {string} params.email - Email to search actorId
     */

  }, {
    key: 'createStudent',
    value: function createStudent() {
      var _ref13 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          email = _ref13.email;

      this.checkArgumentType(email, 'email', 'string');
      return this.post('/students', {
        body: {
          email: email
        },
        bearer: true
      });
    }

    /**
     * Post enrollment
     *
     * @param {Object} params - Data to create new enrollment
     * @param {number} params.programId Id to search programGroupId
     * @param {number} params.groupId Id to search programGroupId
     * @param {string} params.email - Email to search actorId
     * @param {string} params.origin - Origin of the user
     */

  }, {
    key: 'createEnrollment',
    value: function createEnrollment() {
      var _this3 = this;

      var _ref14 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          programId = _ref14.programId,
          groupId = _ref14.groupId,
          email = _ref14.email,
          origin = _ref14.origin;

      this.checkArgumentType(programId, 'programId', 'number');
      this.checkArgumentType(groupId, 'groupId', 'number');
      this.checkArgumentType(email, 'email', 'string');
      if (origin) this.checkArgumentType(origin, 'origin', 'string');

      return this.createStudent({ email: email }).then(function () {
        return _this3.post('/enrollments', {
          body: {
            programId: programId,
            groupId: groupId,
            email: email,
            origin: origin
          },
          bearer: true
        });
      }).catch(function (e) {
        if (e.statusCode !== 409) {
          return Promise.reject(e);
        }

        return _this3.post('/enrollments', {
          body: {
            programId: programId,
            groupId: groupId,
            email: email,
            origin: origin
          },
          bearer: true
        });
      });
    }
  }, {
    key: 'getEnrollments',
    value: function getEnrollments() {
      var _ref15 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          programId = _ref15.programId,
          status = _ref15.status,
          scope = _ref15.scope;

      if (programId) this.checkArgumentType(programId, 'programId', 'number');
      if (status) this.checkArgumentType(status, 'status', 'string');

      if (programId) {
        return this.get('/programs/' + programId + '/enrollments', { bearer: true, params: { scope: scope } });
      }

      return this.get('/enrollments', {
        bearer: true,
        params: {
          status: status
        }
      });
    }
  }, {
    key: 'updateEnrollment',
    value: function updateEnrollment(enrollmentId) {
      var _ref16 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          email = _ref16.email,
          programId = _ref16.programId,
          groupId = _ref16.groupId;

      invariant(isId(enrollmentId), 'Invalid id ' + enrollmentId + ', id must be an integer');

      this.checkArgumentType(email, 'email', 'string');
      this.checkArgumentType(programId, 'programId', 'number');
      this.checkArgumentType(groupId, 'groupId', 'number');

      return this.patch('enrollments/' + enrollmentId, {
        bearer: true,
        body: {
          email: email,
          programId: programId,
          groupId: groupId
        }
      });
    }

    /**
     * Execute a payment (i.e. transfer funds and grant access)
     * @param {Object} args Execute payment arguments
     * @param {string} args.payerId Payer ID returned by the Payment Service Provider
     * @param {number|string} args.paymentId ID of the previously created payment
     * @return {Promise<Object>} Execute payment response
     */

  }, {
    key: 'executePayment',
    value: function executePayment() {
      var _ref17 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          payerId = _ref17.payerId,
          paymentId = _ref17.paymentId;

      this.checkArgumentType(payerId, 'payerId', 'string');
      invariant(isId(paymentId), 'Invalid id ' + paymentId + ', id must be an integer');

      return this.post('/payments/' + paymentId + '/execute', {
        body: {
          payerId: payerId
        },
        bearer: true
      });
    }

    /**
     * Get activities
     * @param {number} activityId - id of the activity
     * @returns {Object} Object of the activity
     */

  }, {
    key: 'getActivity',
    value: function getActivity(activityId) {
      invariant(isId(activityId), 'Invalid id ' + activityId + ', must be an integer');
      return this.get('/activities/' + activityId);
    }

    /**
     * Get activities wich is associated with program
     * @param {number} programId - id of the program
     * @returns {Array<Object>} - returns an array of object activities
     */

  }, {
    key: 'getProgramActivities',
    value: function getProgramActivities(programId) {
      var _ref18 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          name = _ref18.name,
          summary = _ref18.summary,
          order = _ref18.order,
          activityTypeId = _ref18.activityTypeId;

      invariant(isId(programId), 'Invalid id ' + programId + ', must be an integer');

      if (name) this.checkArgumentType(name, 'name', 'string');
      if (summary) this.checkArgumentType(summary, 'summary', 'string');
      if (order) this.checkArgumentType(order, 'order', 'number');
      if (activityTypeId) this.checkArgumentType(activityTypeId, 'activityTypeId', 'number');

      return this.get('/programs/' + programId + '/activities', {
        params: {
          name: name,
          summary: summary,
          order: order,
          activityTypeId: activityTypeId
        }
      });
    }

    /**
     * Get programs with id
     * @param {number} programId - Id for prrogram find
     * @return {Object} - Returns an object with program information
     */

  }, {
    key: 'getProgram',
    value: function getProgram(programId) {
      invariant(isId(programId), 'Invalid id ' + programId + ', must be an integer');

      return this.get('/programs/' + programId);
    }

    /**
     * Create activity progress
     * @param {number}  - Id for prrogram find
     * @return {Object} - Returns an object with program information
     */

  }, {
    key: 'createActivityProgress',
    value: function createActivityProgress() {
      var _ref19 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          activityId = _ref19.activityId,
          enrollmentId = _ref19.enrollmentId,
          time = _ref19.time;

      this.checkArgumentType(time, 'time', 'number');
      this.checkArgumentType(activityId, 'activityId', 'number');
      this.checkArgumentType(enrollmentId, 'enrollmentId', 'number');

      // Use XMLHttpRequest since this data needs to be sent
      // event on page unloads
      var xhr = new XMLHttpRequest();
      var url = join(this.baseURL, '/activity-progress');
      var eventType = window.event && window.event.type;
      var useSync = eventType === 'beforeunload' || eventType === 'unload';

      xhr.open('POST', url, !useSync);
      xhr.setRequestHeader('Content-Type', 'application/json');
      xhr.setRequestHeader('Authorization', 'Bearer ' + this.bearerToken);

      try {
        xhr.send(JSON.stringify({
          payload: {
            time: time,
            activityId: activityId,
            enrollmentId: enrollmentId
          }
        }));
        return true;
      } catch (e) {
        return false;
      }
    }
  }, {
    key: 'getPercent',
    value: function getPercent() {
      var _ref20 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          activityId = _ref20.activityId,
          enrollmentId = _ref20.enrollmentId;

      this.checkArgumentType(activityId, 'activityId', 'number');
      this.checkArgumentType(enrollmentId, 'enrollmentId', 'number');

      return this.get('/activity-progress', {
        params: {
          activityId: activityId,
          enrollmentId: enrollmentId
        },
        bearer: true
      });
    }
  }, {
    key: 'getActivityContent',
    value: function getActivityContent(activityId) {
      var _ref21 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          fileTypeId = _ref21.fileTypeId;

      invariant(isId(activityId), 'Invalid id ' + activityId + ', must be a number');

      if (fileTypeId) this.checkArgumentType(fileTypeId, 'fileTypeId', 'number');
      return this.get('lectures/' + activityId + '/media', {
        params: {
          fileTypeId: fileTypeId
        },
        bearer: true
      });
    }
  }, {
    key: 'getEnrollmentActivities',
    value: function getEnrollmentActivities(enrollmentId) {
      invariant(isId(enrollmentId), 'Invalid id ' + enrollmentId + ', must be a number');
      return this.get('enrollments/' + enrollmentId + '/activities', { bearer: true });
    }
    /**
     * Get categories no empty
     * @param {string} scope - Scope for categories no empty
     * @returns {Object} - Returns an object with categorias
     */

  }, {
    key: 'getCategories',
    value: function getCategories() {
      var _ref22 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          scope = _ref22.scope;

      this.checkArgumentType(scope, 'scope', 'string');

      return this.get('/categories', {
        params: {
          scope: scope
        }
      });
    }

    /**
     * Get programs for categories
     * @param {number} categoryId - Id for programs find by category
     * @param {string} summary - Scope for summary of the programs
     * @returns {Object} - Return an object with programs
     */

  }, {
    key: 'getCategoryPrograms',
    value: function getCategoryPrograms(categoryId) {
      var _ref23 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          scope = _ref23.scope;

      invariant(isId(categoryId), 'Invalid id ' + categoryId + ', must be a number');
      this.checkArgumentType(scope, 'scope', 'string');

      return this.get('/categories/' + categoryId + '/programs', {
        params: {
          scope: scope
        }
      });
    }
  }, {
    key: 'search',
    value: function search() {
      var _ref24 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          q = _ref24.q,
          limit = _ref24.limit,
          page = _ref24.page;

      this.checkArgumentType(q, 'q', 'string');
      this.checkArgumentType(limit, 'limit', 'number');
      this.checkArgumentType(page, 'page', 'number');

      return this.get('/search', {
        params: {
          q: q,
          limit: limit,
          page: page
        }
      });
    }
  }, {
    key: 'updateUser',
    value: function updateUser(userId) {
      var _ref25 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          password = _ref25.password;

      invariant(isId(userId), 'Invalid id ' + userId + ', must be a number');
      this.checkArgumentType(password, 'password', 'string');

      return this.patch('/users/' + userId, {
        bearer: true,
        body: {
          password: password
        }
      });
    }
  }, {
    key: 'getPromos',
    value: function getPromos() {
      var _ref26 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          url = _ref26.url;

      this.checkArgumentType(url, 'url', 'string');

      return this.get('/promos', {
        params: {
          url: url
        }
      });
    }
  }, {
    key: 'getPromosPrograms',
    value: function getPromosPrograms(promoId) {
      invariant(isId(promoId), 'Invalid id ' + promoId + ', must be a number');

      return this.get('/promos/' + promoId + '/programs');
    }
  }, {
    key: 'getPromosGroups',
    value: function getPromosGroups(promoId) {
      var _ref27 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          programId = _ref27.programId;

      invariant(isId(promoId), 'Invalid id ' + promoId + ', must be a number');
      this.checkArgumentType(programId, 'programId', 'number');

      return this.get('promos/' + promoId + '/groups', {
        params: {
          programId: programId
        }
      });
    }

    /**
     * Get an exam by id
     * @param {number} id - Exam id
     */

  }, {
    key: 'getExam',
    value: function getExam(id) {
      var _ref28 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          enrollmentId = _ref28.enrollmentId;

      invariant(isId(id), 'Invalid id ' + id + ', must be a number');
      return this.get('/exams/' + id + '/find', { bearer: true, params: { enrollmentId: enrollmentId } });
    }

    /**
     * Mark an exam as started
     * @param {number} id - Id of the exam to answer
     * @param {Object} options - Request options
     * @param {number} options.enrollmentId - Enrollment starting the exam
     */

  }, {
    key: 'startExam',
    value: function startExam(id) {
      var _ref29 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          enrollmentId = _ref29.enrollmentId;

      invariant(isId(id), 'Invalid id ' + id + ', must be a number');
      invariant(isId(enrollmentId), 'Invalid id ' + enrollmentId + ', must be a number');

      return this.post('/exams/' + id + '/start', {
        bearer: true,
        body: {
          enrollmentId: enrollmentId
        }
      });
    }

    /**
     * Answer a question from an exam
     * @param {number} id - Id of the exam
     * @param {Object} options - Request options
     * @param {number} options.answerId Id of the answer
     * @param {number} options.questionId Id of the question
     * @param {number} options.enrollmentId - Id of the enrollment answering the question
     */

  }, {
    key: 'answerExamQuestion',
    value: function answerExamQuestion(id) {
      var _ref30 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
          enrollmentId = _ref30.enrollmentId,
          answerId = _ref30.answerId,
          questionId = _ref30.questionId;

      invariant(isId(id), 'Invalid id ' + id + ', must be a number');
      invariant(isId(enrollmentId), '\xCCnvalid enrollmentId, ' + enrollmentId + ' must be a number');
      return this.patch('/exams/' + id + '/answer', {
        bearer: true,
        body: {
          answerId: answerId,
          questionId: questionId,
          enrollmentId: enrollmentId
        }
      });
    }
  }, {
    key: 'gradeExam',
    value: function gradeExam(id, _ref31) {
      var enrollmentId = _ref31.enrollmentId;

      return this.patch('/exams/' + id + '/grade', {
        bearer: true,
        body: {
          enrollmentId: enrollmentId
        }
      });
    }
  }, {
    key: 'bearer',
    set: function set(value) {
      this.checkArgumentType(value, 'bearer', 'string');
      this.bearerToken = value;
    },
    get: function get() {
      return this.bearerToken;
    }
  }]);

  return AulaMejorClient;
}(Bound);


AulaMejorClient.prototype.ENROLLMENT_IS_PENDING_ERROR = 'EnrollmentIsPending';
AulaMejorClient.prototype.ENROLLMENT_IS_PAID_ERROR = 'EnrollmentIsPaid';

exports.default = AulaMejorClient;
exports.joinURL = join;
